require_relative 'configuration'

module BackgroundJobs
  class UnknownBackgroundJobsQueueStrategy < RuntimeError
    def initialize(strategy_name)
      @strategy_name = strategy_name
    end

    def to_s
      "Unknown strategy '#{@strategy_name}' doesnt exist!"
    end
  end

  class JobQueueFactory
    def self.build
      to_class(Configuration.instance.strategy)
    end

    private

    def self.to_class(strategy_name)
      case strategy_name
      when 'sidekiq'
        require 'background_jobs/strategies/sidekiq/job_queue'
        Sidekiq::JobQueue.new

      when 'direct_call'
        require 'background_jobs/strategies/direct_call/job_queue'
        DirectCall::JobQueue.new

      else
        raise UnknownBackgroundJobsQueueStrategy.new(strategy_name)
      end 
    end
  end
end
