require 'singleton'

module BackgroundJobs
  class UnknownJob < RuntimeError
    def initialize(job_name)
      @job_name = job_name
    end

    def to_s
      "Unknown job '#{@job_name}'!"
    end
  end

  class JobMetadata
    attr_reader :name, :type, :clazz, :options

    def initialize(name, type, clazz, options = nil)
      @name, @type, @clazz, @options = name, type, clazz, options
    end
  end

  class JobRegistry
    include Singleton

    attr_writer :register_callback

    def register_job(job_metadata)
      jobs[job_metadata.name] = job_metadata
      
      @register_callback.call(job_metadata) if @register_callback
    end

    def get_class!(job_name)
      raise UnknownJob.new(job_name) unless job_metadata = get(job_name)

      job_metadata.clazz
    end

    def get_type(job_name)
      raise UnknownJob.new(job_name) unless job_metadata = get(job_name)

      job_metadata.type
    end

    private

    def get(job_name)
      jobs[job_name]
    end

    def jobs
      @jobs ||= {}
    end
  end

  def self.register_job(job_name, job_type, job_class, options = {})
    job_registry.register_job JobMetadata.new(job_name, job_type, job_class, options)
  end

  private

  def self.job_registry
    JobRegistry.instance
  end
end