require 'sidekiq'
require 'background_jobs/job_registry'

module BackgroundJobs
  module Sidekiq
    def self.setup(options)
      ::Sidekiq.configure_server do |config|
        config.redis = options[:redis] if options[:redis]
      end

      BackgroundJobs.job_registry.register_callback = method(:register_job_callback)
    end

    def self.register_job_callback(job_metadata)
      options = job_metadata.options

      BackgroundJobs::QueueService.enqueue(
        job_metadata.name, [], :in => options[:schedule], :unique => true
      ) if options[:schedule]
    end

    private

    def self.is_scheduled?(job_name)
      entries = ::Sidekiq::ScheduledSet.new.entries
      job_names = entries.map(&:item).map {|item| item['args'].first }

      job_names.include?(job_name.to_s)
    end
  end
end